@@ -7,15 +7,20 @@ use Fcntl qw( :DEFAULT :seek ) ;
 use Symbol ;
 
 use base 'Exporter' ;
-use vars qw( %EXPORT_TAGS @EXPORT_OK $VERSION  @EXPORT) ;
+use vars qw( %EXPORT_TAGS @EXPORT_OK $VERSION @EXPORT ) ;
 
 %EXPORT_TAGS = ( 'all' => [
 	qw( read_file write_file overwrite_file append_file read_dir ) ] ) ;
 
-#@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 @EXPORT = ( @{ $EXPORT_TAGS{'all'} } );
+@EXPORT_OK = qw( slurp ) ;
 
-$VERSION = '9999.04';
+$VERSION = '9999.08';
+
+{
+no strict 'refs' ;
+*slurp = \&read_file ;
+}
 
 sub read_file {
 
@@ -40,10 +45,27 @@ sub read_file {
 		$read_fh = $file_name ;
 		$blk_size = $args{'blk_size'} || 1024 * 1024 ;
 		$size_left = $blk_size ;
+
+# DEEP DARK MAGIC. this checks the UNTAINT IO flag of a
+# glob/handle. only the DATA handle is untainted (since it is from
+# trusted data in the source file). this allows us to test if this is
+# the DATA handle and then to do a sysseek to make sure it gets
+# slurped correctly. on some systems, the buffered i/o pointer is not
+# left at the same place as the fd pointer. this sysseek makes them
+# the same so slurping with sysread will work.
+
+		require B ;
+		if ( B::svref_2object( $read_fh )->IO->IoFLAGS & 16 ) {
+
+# set the seek position to the current tell.
+
+			sysseek( $read_fh, tell( $read_fh ), SEEK_SET ) ||
+				croak "sysseek $!" ;
+		}
 	}
 	else {
 
-# a regular file. set the sysopne mode
+# a regular file. set the sysopen mode
 
 		my $mode = O_RDONLY ;
 		$mode |= O_BINARY if $args{'binmode'} ;
@@ -59,6 +81,12 @@ sub read_file {
 # get the size of the file for use in the read loop
 
 		$size_left = -s $read_fh ;
+
+		unless( $size_left ) {
+
+			$blk_size = $args{'blk_size'} || 1024 * 1024 ;
+			$size_left = $blk_size ;
+		}
 	}
 
 # infinite read loop. we exit when we are done slurping
@@ -97,11 +125,18 @@ sub read_file {
 
 # handle wanting to return an array ref of lines
 
-	return [ split( m|(?<=$/)|, ${$buf_ref} ) ] if $args{'array_ref'}  ;
+	my $sep = $/ ;
+	$sep = '\n\n+' if defined $sep && $sep eq '' ;
+
+#	return [ split( m|(?<=$sep)|, ${$buf_ref} ) ] if $args{'array_ref'}  ;
+	return [ length(${$buf_ref}) ? ${$buf_ref} =~ /(.*?$sep|.+)/sg : () ]
+		if $args{'array_ref'}  ;
 
 # handle wanting a list of lines (normal list context)
 
-	return split( m|(?<=$/)|, ${$buf_ref} ) if wantarray ;
+#	return split( m|(?<=$sep)|, ${$buf_ref} ) if wantarray ;
+	return length(${$buf_ref}) ? ${$buf_ref} =~ /(.*?$sep|.+)/sg : ()
+		if wantarray ;
 
 # handle wanting to return an scalar ref to the slurped text
 
@@ -220,6 +255,7 @@ sub write_file {
 	} while( $size_left > 0 ) ;
 
 # we truncate regular files in case we overwrite a long file with a shorter file
+# so seek to the current position to get it (same as tell()).
 
 	truncate( $write_fh,
 		  sysseek( $write_fh, 0, SEEK_CUR ) ) unless $no_truncate ;
@@ -265,7 +301,7 @@ sub append_file {
 	goto &write_file
 }
 
-# simple wrapper around opendir/readdir
+# basic wrapper around opendir/readdir
 
 sub read_dir {
 
@@ -275,18 +311,21 @@ sub read_dir {
 
 	local(*DIRH);
 
-# open the dir
+# open the dir and handle any errors
 
-	if ( opendir( DIRH, $dir ) ) {
+	unless ( opendir( DIRH, $dir ) ) {
 
-# return all the directory entries but . and ..
-		return grep( $_ ne "." && $_ ne "..", readdir(DIRH));
+		@_ = ( \%args, "read_dir '$dir' - opendir: $!" ) ;
+		goto &error ;
 	}
 
-# handle the error
+	my @dir_entries = readdir(DIRH) ;
 
-	@_ = ( \%args, "read_dir '$dir' - opendir: $!" ) ;
-	goto &error ;
+	@dir_entries = grep( $_ ne "." && $_ ne "..", @dir_entries )
+		unless $args{'keep_dot_dot'} ;
+
+	return @dir_entries if wantarray ;
+	return \@dir_entries ;
 }
 
 # error handling section
@@ -342,6 +381,11 @@ File::Slurp - Efficient Reading/Writing of Complete Files
 
   write_file( 'filename', @lines ) ;
 
+  use File::Slurp qw( slurp ) ;
+
+  my $text = slurp( 'filename' ) ;
+
+
 =head1 DESCRIPTION
 
 This module provides subs that allow you to read or write entire files
@@ -350,17 +394,16 @@ flexible ways to pass in or get the file contents and to be very
 efficient.  There is also a sub to read in all the files in a
 directory other than C<.> and C<..>
 
-Note that these slurp/spew subs work only for files and not for pipes
-or stdio. If you want to slurp the latter, use the standard techniques
-such as setting $/ to undef, reading <> in a list context, or printing
-all you want to STDOUT.
+These slurp/spew subs work for files, pipes and
+sockets, and stdio, pseudo-files, and DATA.
 
 =head2 B<read_file>
 
 This sub reads in an entire file and returns its contents to the
 caller. In list context it will return a list of lines (using the
-current value of $/ as the separator. In scalar context it returns the
-entire file as a single scalar.
+current value of $/ as the separator including support for paragraph
+mode when it is set to ''). In scalar context it returns the entire
+file as a single scalar.
 
   my $text = read_file( 'filename' ) ;
   my @lines = read_file( 'filename' ) ;
@@ -377,19 +420,13 @@ for an example that does C<open( '-|' )> and child process spews data
 to the parant which slurps it in.  All of the options that control how
 the data is returned to the caller still work in this case.
 
-NOTE: the C<DATA> handle works with C<read_file> but it needs a small
-workaround. The issue is that DATA is opened with buffered I/O and
-read_file doesn't use that. So the buffered DATA handle is at the
-correct place but the unbuffered one is at the real EOF. The
-workaround is to get the buffered seek location and set the unbuffered
-seek pointer like this:
-
-	use Fcntl qw( :seek ) ;
-	sysseek( DATA, tell( \*DATA ), SEEK_SET ) ;
-	my $slurp_text = read_file( \*DATA ) ;
-
-See the t/handle.t test for more examples of this.
+NOTE: as of version 9999.06, read_file works correctly on the C<DATA>
+handle. It used to need a sysseek workaround but that is now handled
+when needed by the module itself
 
+You can optionally request that C<slurp()> is exported to your code. This
+is an alias for read_file and is meant to be forward compatible with
+Perl 6 (which will have slurp() built-in).
 
 The options are:
 
@@ -408,7 +445,7 @@ file modes.
 
 If this boolean option is set, the return value (only in scalar
 context) will be an array reference which contains the lines of the
-slurped file. The following two calls are equivilent:
+slurped file. The following two calls are equivalent:
 
 	my $lines_ref = read_file( $bin_file, array_ref => 1 ) ;
 	my $lines_ref = [ read_file( $bin_file ) ] ;
@@ -569,16 +606,31 @@ documentation. These calls are equivilent:
 =head2 read_dir
 
 This sub reads all the file names from directory and returns them to
-the caller but C<.> and C<..> are removed.
+the caller but C<.> and C<..> are removed by default.
 
 	my @files = read_dir( '/path/to/dir' ) ;
 
 It croaks if it cannot open the directory.
 
+In a list context C<read_dir> returns a list of the entries in the
+directory. In a scalar context it returns an array reference which has
+the entries.
+
+=head3 keep_dot_dot
+
+If this boolean option is set, C<.> and C<..> are not removed from the
+list of files.
+
+	my @all_files = read_dir( '/path/to/dir', keep_dot_dot => 1 ) ;
+
 =head2 EXPORT
 
   read_file write_file overwrite_file append_file read_dir
 
+=head2 SEE ALSO
+
+An article on file slurping 
+
 =head1 AUTHOR
 
 Uri Guttman, E<lt>uri@stemsystems.comE<gt>
